@@ -1,5 +1,5 @@
 /* Rational - Rational number class with overflow detection
-   Copyright (C) 2012-2024 Antonio Diaz Diaz.
+   Copyright (C) 2012-2025 Antonio Diaz Diaz.
 
    This program is free software. Redistribution and use in source and
    binary forms, with or without modification, are permitted provided
@@ -38,8 +38,9 @@
 namespace {
 
 const char * const program_name = "rational";
-const char * const program_year = "2024";
+const char * const program_year = "2025";
 const char * invocation_name = program_name;		// default value
+int verbosity = 0;
 
 
 void show_help()
@@ -73,6 +74,7 @@ void show_help()
   std::printf( "\nOptions:\n"
                "  -h, --help                 display this help and exit\n"
                "  -V, --version              output version information and exit\n"
+               "  -a, --approximate=<n.n>    calculate approximations of <n.n>\n"
                "  -g, --golden-ratio         calculate approximations of (1 + sqrt(5)) / 2\n"
                "  -p, --pi                   calculate approximations of pi\n"
                "  -q, --quiet                quiet operation\n"
@@ -95,6 +97,15 @@ void show_version()
   }
 
 
+void show_option_error( const char * const arg, const char * const msg,
+                        const char * const option_name )
+  {
+  if( verbosity >= 0 )
+    std::fprintf( stderr, "%s: '%s': %s option '%s'.\n",
+                  program_name, arg, msg, option_name );
+  }
+
+
 void show_error( const char * const msg, const int errcode = 0,
                  const bool help = false )
   {
@@ -119,8 +130,8 @@ void approximate_golden_ratio()
   {
   const std::string target( "1.618033988749894848204586834365638117720" );
   const int tlen = target.size();
-  int max_len = 1;		// number of chars identical to target
   int n = 1, d = 1;	// Fibonacci F(n+1)/F(n) converges to the golden ratio
+  int max_len = 0;		// number of chars identical to target
 
   std::printf( "target = %s\n", target.c_str() );
   std::fputs( "correct\ndecimals                  value\n", stdout );
@@ -130,14 +141,14 @@ void approximate_golden_ratio()
     int i = 2;
     for( ; i < tlen; ++i )
       {
-      rest *= 10; const int dif = ( rest / d ) + '0' - target[i]; rest %= d;
+      rest *= 10; const int dif = rest / d + '0' - target[i]; rest %= d;
       if( dif != 0 ) break;
       }
     if( i > max_len )
       {
       const Rational r( n, d );
       max_len = i;
-      std::printf( "%2d      %s (%s)\n", i - 2,
+      std::printf( "%2d       %s (%s)\n", i - 2,
                    r.to_decimal( 1, -(tlen - 2) ).c_str(),
                    r.to_fraction().c_str() );
       std::fflush( stdout );
@@ -156,18 +167,74 @@ inline long long compare_fraction( const int n1, const int d1,
   { return (long long)n1 * d2 - (long long)n2 * d1; }
 
 
+bool approximate_decimal( const std::string & target )
+  {
+  const int tlen = target.size();
+  int ip = 0;					// integer part of target
+  int dpos = 0;					// pos of first decimal digit
+  while( dpos < tlen && std::isdigit( target[dpos] ) )
+    {
+    if( ( INT_MAX - (target[dpos] - '0') ) / 10 < ip ) return false;
+    ip = ip * 10 + (target[dpos] - '0'); ++dpos;
+    }
+  if( dpos + 1 >= tlen || target[dpos++] != '.' ) return false;
+  for( int i = dpos; i < tlen; ++i )
+    if( !std::isdigit( target[i] ) ) return false;
+  unsigned n = ip, d = 1;	// initialize n/d to the integer part of target
+  int nmin = n - 1, dmin = d;			// lower limit of search
+  int nmax = n + (n < INT_MAX), dmax = d;	// upper limit of search
+  int max_len = 0;		// number of chars identical to target
+  int dif = 0;
+
+  std::printf( "target = %s\n", target.c_str() );
+  std::fputs( "correct\ndecimals                  value\n", stdout );
+  Rational r;
+  if( r.parse( target.c_str() ) && r.to_decimal( 1, tlen - dpos ) == target )
+    { std::printf( "%2d       %s (%s)\n", tlen - dpos,
+                   r.to_decimal( 1, -(tlen - dpos) ).c_str(),
+                   r.to_fraction().c_str() ); std::fflush( stdout );
+      return true; }
+  while( n <= INT_MAX && d <= INT_MAX )
+    {
+    if( compare_fraction( n, d, nmax, dmax ) >= 0 ) { n += ip; ++d; continue; }
+    if( compare_fraction( n, d, nmin, dmin ) <= 0 ) { ++n; continue; }
+    long long rest = n % d;
+    int i = dpos;
+    for( ; i < tlen; ++i )
+      {
+      rest *= 10; dif = rest / d + '0' - target[i]; rest %= d;
+      if( dif != 0 ) break;
+      }
+    if( i > max_len )
+      {
+      const Rational r( n, d );
+      max_len = i;
+      std::printf( "%2d       %s (%s)\n", i - dpos,
+                   r.to_decimal( 1, -(tlen - dpos) ).c_str(),
+                   r.to_fraction().c_str() );
+      std::fflush( stdout );
+      if( max_len >= tlen ) break;
+      }
+    if( dif < 0 ) { nmin = n; dmin = d; ++n; }	// increase lower limit
+    else { nmax = n; dmax = d; n += ip; ++d; }	// decrease upper limit
+    }
+  return true;
+  }
+
+
 void approximate_pi()
   {
   const std::string target( "3.141592653589793238462643383279502884197" );
   const int tlen = target.size();
-  int max_len = 1;		// number of chars identical to target
-  int n = 3, d = 1, dif = 0;	// initialize n/d to the integer part of target
+  unsigned n = 3, d = 1;	// initialize n/d to the integer part of target
   int nmin = 29, dmin = 10;	// lower limit of search (2.9)
   int nmax = 32, dmax = 10;	// upper limit of search (3.2)
+  int max_len = 0;		// number of chars identical to target
+  int dif = 0;
 
   std::printf( "target = %s\n", target.c_str() );
   std::fputs( "correct\ndecimals                  value\n", stdout );
-  while( n <= INT_MAX - 3 )
+  while( n <= INT_MAX )
     {
     if( compare_fraction( n, d, nmax, dmax ) >= 0 ) { n += 3; ++d; continue; }
     if( compare_fraction( n, d, nmin, dmin ) <= 0 ) { ++n; continue; }
@@ -175,14 +242,14 @@ void approximate_pi()
     int i = 2;
     for( ; i < tlen; ++i )
       {
-      rest *= 10; dif = ( rest / d ) + '0' - target[i]; rest %= d;
+      rest *= 10; dif = rest / d + '0' - target[i]; rest %= d;
       if( dif != 0 ) break;
       }
     if( i > max_len )
       {
       const Rational r( n, d );
       max_len = i;
-      std::printf( "%2d      %s (%s)\n", i - 2,
+      std::printf( "%2d       %s (%s)\n", i - 2,
                    r.to_decimal( 1, -(tlen - 2) ).c_str(),
                    r.to_fraction().c_str() );
       std::fflush( stdout );
@@ -194,9 +261,10 @@ void approximate_pi()
   }
 
 
-void approximate_sqrt( const Rational target )
+bool approximate_sqrt( const Rational target )
   {
-  // works for targets from 1/65536 to 462409
+  // works for targets from 1 / 65536 to 462409
+  if( target < Rational( 1, 65536 ) || target > 462409 ) return false;
   Rational best_error( target );		// for 0 < target < 1
   if( target >= 46226 ) best_error = 10;
   else if( target >= 1 ) best_error.assign( 1, ( target >= 15047 ) ? 1 : 10 );
@@ -220,7 +288,7 @@ void approximate_sqrt( const Rational target )
       best_error = error.abs();
       std::printf( "%-23s    %-19s    %-12s (%s)\n",
                    error.to_decimal( 2, -2 * prec ).c_str(),
-                   value2.to_decimal( 1, -prec ).c_str(),
+                   value2.to_decimal( 1, -prec - 6 ).c_str(),
                    value.to_decimal( 1, -prec ).c_str(),
                    value.to_fraction().c_str() );
       std::fflush( stdout );
@@ -229,24 +297,25 @@ void approximate_sqrt( const Rational target )
     if( value2 < target ) { if( ++n >= INT_MAX ) break; }
     else if( ++d >= INT_MAX ) break;
     }
+  return true;
   }
 
 
-int run_tests( const int level, const bool verbose )
+bool run_tests( const int level )
   {
-  int retval = 0;
+  bool fail = false;
 
   // relational on error test
   {
   const Rational r( 1, 0 );
   if( !r.error() )
     { std::fputs( "error: Rational( 1, 0 ) not rejected.\n", stderr );
-      retval = 1; }
+      fail = true; }
   if( r == 0 || r == 1 || r == INT_MAX || r == -INT_MAX ||
       !(r != 0) || !(r != 1) || !(r != INT_MAX) || !(r != -INT_MAX) ||
       r > -INT_MAX || r >= -INT_MAX || r < INT_MAX || r <= INT_MAX )
     { std::fputs( "error: bad relational result on error.\n", stderr );
-      retval = 1; }
+      fail = true; }
   }
 
   // ( num, 1 ) test
@@ -254,23 +323,23 @@ int run_tests( const int level, const bool verbose )
   const Rational r0( INT_MAX, 1 );
   if( r0 != INT_MAX )
     { std::fputs( "error: Rational( INT_MAX, 1 ) not accepted.\n", stderr );
-      retval = 1; }
+      fail = true; }
   Rational r( INT_MIN, 1 );
   if( !r.error() )
     { std::fputs( "error: Rational( INT_MIN, 1 ) not rejected.\n", stderr );
-      retval = 1; }
+      fail = true; }
   r.assign( INT_MAX, 1 );
   if( r != INT_MAX || r != r0 )
     { std::fputs( "error: r.assign( INT_MAX, 1 ) not accepted.\n", stderr );
-      retval = 1; }
+      fail = true; }
   r.assign( INT_MIN, 1 );
   if( !r.error() )
     { std::fputs( "error: r.assign( INT_MIN, 1 ) not rejected.\n", stderr );
-      retval = 1; }
+      fail = true; }
   r.assign( -INT_MAX, 1 );
   if( r != -INT_MAX || r != -r0 || -r != r0 )
     { std::fputs( "error: r.assign( -INT_MAX, 1 ) not accepted.\n", stderr );
-      retval = 1; }
+      fail = true; }
   }
 
   // ( 1, num ) test
@@ -278,24 +347,24 @@ int run_tests( const int level, const bool verbose )
   const Rational r0( 1, INT_MAX );
   if( r0.numerator() != 1 || r0.denominator() != INT_MAX )
     { std::fputs( "error: Rational( 1, INT_MAX ) not accepted.\n", stderr );
-      retval = 1; }
+      fail = true; }
   Rational r( 1, INT_MIN );
   if( !r.error() )
     { std::fputs( "error: Rational( 1, INT_MIN ) not rejected.\n", stderr );
-      retval = 1; }
+      fail = true; }
   r.assign( 1, INT_MAX );
   if( r.numerator() != 1 || r.denominator() != INT_MAX || r != r0 )
     { std::fputs( "error: r.assign( 1, INT_MAX ) not accepted.\n", stderr );
-      retval = 1; }
+      fail = true; }
   r.assign( 1, INT_MIN );
   if( !r.error() )
     { std::fputs( "error: r.assign( 1, INT_MIN ) not rejected.\n", stderr );
-      retval = 1; }
+      fail = true; }
   r.assign( 1, -INT_MAX );
   if( r.numerator() != -1 || r.denominator() != INT_MAX ||
       r != -r0 || -r != r0 )
     { std::fputs( "error: r.assign( 1, -INT_MAX ) not accepted.\n", stderr );
-      retval = 1; }
+      fail = true; }
   }
 
   // ( num, num ) test
@@ -303,23 +372,23 @@ int run_tests( const int level, const bool verbose )
   const Rational r0( INT_MAX, INT_MAX );
   if( r0 != 1 )
     { std::fputs( "error: Rational( INT_MAX, INT_MAX ) not accepted.\n", stderr );
-      retval = 1; }
+      fail = true; }
   Rational r( INT_MIN, INT_MIN );
   if( r != 1 || r != r0 )
     { std::fputs( "error: Rational( INT_MIN, INT_MIN ) not accepted.\n", stderr );
-      retval = 1; }
+      fail = true; }
   r.assign( INT_MAX, INT_MAX );
   if( r != 1 || r != r0 )
     { std::fputs( "error: r.assign( INT_MAX, INT_MAX ) not accepted.\n", stderr );
-      retval = 1; }
+      fail = true; }
   r.assign( INT_MIN, INT_MIN );
   if( r != 1 || r != r0 )
     { std::fputs( "error: r.assign( INT_MIN, INT_MIN ) not accepted.\n", stderr );
-      retval = 1; }
+      fail = true; }
   r.assign( INT_MAX, -INT_MAX );
   if( r != -1 || r != -r0 || -r != r0 )
     { std::fputs( "error: r.assign( INT_MAX, -INT_MAX ) not accepted.\n", stderr );
-      retval = 1; }
+      fail = true; }
   }
 
   // ( num ) test
@@ -327,30 +396,30 @@ int run_tests( const int level, const bool verbose )
   const Rational r0( INT_MAX );
   if( r0 != INT_MAX )
     { std::fputs( "error: Rational( INT_MAX ) not accepted.\n", stderr );
-      retval = 1; }
+      fail = true; }
   Rational r( INT_MIN );
   if( !r.error() )
     { std::fputs( "error: Rational( INT_MIN ) not rejected.\n", stderr );
-      retval = 1; }
+      fail = true; }
   r = INT_MAX;
   if( r != INT_MAX || r != r0 )
     { std::fputs( "error: ( r = INT_MAX ) not accepted.\n", stderr );
-      retval = 1; }
+      fail = true; }
   r = INT_MIN;
   if( !r.error() )
     { std::fputs( "error: ( r = INT_MIN ) not rejected.\n", stderr );
-      retval = 1; }
+      fail = true; }
   r = -INT_MAX;
   if( r != -INT_MAX || r != -r0 || -r != r0 )
     { std::fputs( "error: ( r = -INT_MAX ) not accepted.\n", stderr );
-      retval = 1; }
+      fail = true; }
   r = 0;
   if( r != 0 || r != r0 - r0 || r != -r )
-    { std::fputs( "error: ( r = 0 ) not accepted.\n", stderr ); retval = 1; }
+    { std::fputs( "error: ( r = 0 ) not accepted.\n", stderr ); fail = true; }
   r = r.inverse();
   if( !r.error() )
     { std::fputs( "error: inverse( 0 ) not rejected.\n", stderr );
-      retval = 1; }
+      fail = true; }
   }
 
   // zero test
@@ -358,25 +427,25 @@ int run_tests( const int level, const bool verbose )
   const Rational r1( 0, 1 );
   const Rational r2( 0 );
   if( r1 != r2 || r1 != -r2 || r1 + r2 != 0 || r1 - r2 != 0 || r1 * r2 != 0 )
-    { std::fputs( "error: ( 0, 1 ) != ( 0 )\n", stderr ); retval = 1; }
+    { std::fputs( "error: ( 0, 1 ) != ( 0 )\n", stderr ); fail = true; }
   const Rational r3( r2.inverse() );
   const Rational r4( 1, 0 );
   if( !r3.error() || !r4.error() || r3.numerator() != r4.numerator() ||
       r3.denominator() != r4.denominator() )
     { std::fprintf( stderr, "error: inverse( 0 ) [%d,%d] != Rational( 1, 0 ) [%d,%d]\n",
       r3.numerator(), r3.denominator(), r4.numerator(), r4.denominator() );
-      retval = 1; }
+      fail = true; }
   Rational r;
   if( r.numerator() != r2.numerator() || r.denominator() != r2.denominator() )
     { std::fprintf( stderr, "error: Rational() [%d,%d] != Rational( 0 ) [%d,%d]\n",
       r.numerator(), r2.denominator(), r.numerator(), r2.denominator() );
-      retval = 1; }
+      fail = true; }
   r.assign( 1, 0 );
   if( !r.error() || r.numerator() != r4.numerator() ||
       r.denominator() != r4.denominator() )
     { std::fprintf( stderr, "error: assign( 1, 0 ) [%d,%d] != Rational( 1, 0 ) [%d,%d]\n",
       r.numerator(), r.denominator(), r4.numerator(), r4.denominator() );
-      retval = 1; }
+      fail = true; }
   }
 
   // intermediate overflow test
@@ -386,8 +455,8 @@ int run_tests( const int level, const bool verbose )
   if( r1 != -r2 || -r1 != r2 || r1 + r2 != 0 ||
       r1 + r1 !=  INT_MAX || 2 * r1 !=  INT_MAX ||
       r2 + r2 != -INT_MAX || 2 * r2 != -INT_MAX )
-    { std::fputs( "error: INT_MAX/2 + INT_MAX/2 != INT_MAX\n", stderr );
-      retval = 1; }
+    { std::fputs( "error: INT_MAX / 2 + INT_MAX / 2 != INT_MAX\n", stderr );
+      fail = true; }
   }
 
   // relational test
@@ -398,22 +467,22 @@ int run_tests( const int level, const bool verbose )
   const Rational r2( 2 );
   if( rm1 < rm1 || r0 < r0 || r1 < r1 || r2 < r2 )
     { std::fputs( "error: bad relational result for 'a < a'.\n", stderr );
-      retval = 1; }
+      fail = true; }
   if( r2 < r1 || r1 < r0 || r0 < rm1 )
     { std::fputs( "error: bad relational result for 'a < b'.\n", stderr );
-      retval = 1; }
+      fail = true; }
   if( r2 <= r1 || r1 <= r0 || r0 <= rm1 )
     { std::fputs( "error: bad relational result for 'a <= b'.\n", stderr );
-      retval = 1; }
+      fail = true; }
   if( rm1 > rm1 || r0 > r0 || r1 > r1 || r2 > r2 )
     { std::fputs( "error: bad relational result for 'a > a'.\n", stderr );
-      retval = 1; }
+      fail = true; }
   if( rm1 > r0 || r0 > r1 || r1 > r2 )
     { std::fputs( "error: bad relational result for 'a > b'.\n", stderr );
-      retval = 1; }
+      fail = true; }
   if( rm1 >= r0 || r0 >= r1 || r1 >= r2 )
     { std::fputs( "error: bad relational result for 'a >= b'.\n", stderr );
-      retval = 1; }
+      fail = true; }
   }
 
   const int sum_delta = ( 10 - level ) * 1000;
@@ -434,7 +503,7 @@ int run_tests( const int level, const bool verbose )
         +r1 + r3 != 0 || -r1 - r3 != 0 ||  n2 != r1 - r3 ||
         +r2 + r4 != 0 || -r2 - r4 != 0 || -n2 != r4 - r2 )
       { std::fprintf( stderr, "error: ( 2 * %d, 2 ) != %d\n", n, n );
-        retval = 1; break; }
+        fail = true; break; }
     }
 
   // multiplication/division test
@@ -444,7 +513,7 @@ int run_tests( const int level, const bool verbose )
     const Rational r( n );
     if( r * n / r != n || r / n * r != n || r * r / r != n || r * n / r != r )
       { std::fprintf( stderr, "error: %d * %d / %d != %d\n", n, n, n, n );
-        retval = 1; break; }
+        fail = true; break; }
     }
 
   // multiplication by inverse test
@@ -454,8 +523,8 @@ int run_tests( const int level, const bool verbose )
     const Rational r1( n );
     const Rational r2( r1.inverse() );
     if( r1 * r2 != 1 )
-      { std::fprintf( stderr, "error: %d * 1/%d != 1\n", n, n );
-        retval = 1; break; }
+      { std::fprintf( stderr, "error: %d * (1 / %d) != 1\n", n, n );
+        fail = true; break; }
     }
 
   // multiplication by INT_MAX inverse test
@@ -467,22 +536,22 @@ int run_tests( const int level, const bool verbose )
 
   if( r1 * r2 !=  1 || r2 * r1 !=  1 || r3 * r4 !=  1 || r4 * r3 !=  1 ||
       r1 * r4 != -1 || r4 * r1 != -1 || r2 * r3 != -1 || r3 * r2 != -1 )
-    { std::fprintf( stderr, "error: %d * 1/%d != 1\n", INT_MAX, INT_MAX );
-      retval = 1; }
+    { std::fprintf( stderr, "error: %d * (1 / %d) != 1\n", INT_MAX, INT_MAX );
+      fail = true; }
 
   if( r1 * r1.inverse() !=  1 || r2 * r2.inverse() !=  1 ||
       r3 * r3.inverse() !=  1 || r4 * r4.inverse() !=  1 ||
       r1 * r3.inverse() != -1 || r2 * r4.inverse() != -1 ||
       r3 * r1.inverse() != -1 || r4 * r2.inverse() != -1 )
     { std::fprintf( stderr, "error: %d * inverse( %d ) != 1\n", INT_MAX, INT_MAX );
-      retval = 1; }
+      fail = true; }
 
   if( r1 * r4.abs() !=  1 || r2 * r3.abs() !=  1 ||
       r4 * r1.abs() != -1 || r3 * r2.abs() != -1 ||
       r3 * r4.abs() != -1 || r4 * r3.abs() != -1 ||
       r3.abs() * r4.abs() != 1 || r4.abs() * r3.abs() != 1 )
-    { std::fprintf( stderr, "error: %d * abs( 1/%d ) != 1\n", INT_MAX, INT_MAX );
-      retval = 1; }
+    { std::fprintf( stderr, "error: %d * abs( 1 / %d ) != 1\n", INT_MAX, INT_MAX );
+      fail = true; }
   }
 
   // operation chain test
@@ -497,14 +566,14 @@ int run_tests( const int level, const bool verbose )
   r--; r /= rn; --r; r /= n; r -= rn; +r -= n;
   if( r != r0 || ri != 2000002001 )
     { std::fprintf( stderr, "error: failed operation chain up ( ri == %s )\n",
-                    ri.to_decimal( 1, -100 ).c_str() ); retval = 1; }
+                    ri.to_decimal( 1, -100 ).c_str() ); fail = true; }
 
   r -= n; r -= rn; r *= n; --r; r *= rn; r--;
   ri = r;
   r++; r /= rn; ++r; r /= n; r += rn; r += n;
   if( r != r0 || ri != -2000000001 )
     { std::fprintf( stderr, "error: failed operation chain down ( ri == %s )\n",
-                    ri.to_decimal( 1, -100 ).c_str() ); retval = 1; }
+                    ri.to_decimal( 1, -100 ).c_str() ); fail = true; }
   }
 
   // round test
@@ -517,17 +586,17 @@ int run_tests( const int level, const bool verbose )
         (rf += ra).round() != i || ( i != 0 && (rf += ra).round() != i ) )
       { std::fprintf( stderr, "error: (%s).round() == %d != %d )\n",
                       rf.to_decimal( 1, -100 ).c_str(), rf.round(), i );
-        retval = 1; break; }
+        fail = true; break; }
     }
   if( rf != Rational( 12425, 100 ) )
     { std::fprintf( stderr, "error: bad final value in round() test ( rf == %s )\n",
-                    rf.to_decimal( 1, -100 ).c_str() ); retval = 1; }
+                    rf.to_decimal( 1, -100 ).c_str() ); fail = true; }
   }
 
   // trunc test
   {
   Rational rf( -125 );
-  const Rational ra( 1, 4 );			//    0.25
+  const Rational ra( 1, 4 );				// 0.25
   for( int i = -124; i <= 124; ++i )
     {
     if( (rf += ra).trunc() != i || (rf += ra).trunc() != i ||
@@ -536,11 +605,11 @@ int run_tests( const int level, const bool verbose )
           (rf += ra).trunc() != i || (rf += ra).trunc() != i ) ) )
       { std::fprintf( stderr, "error: (%s).trunc() == %d != %d )\n",
                       rf.to_decimal( 1, -100 ).c_str(), rf.trunc(), i );
-        retval = 1; break; }
+        fail = true; break; }
     }
   if( rf != Rational( 12475, 100 ) )
     { std::fprintf( stderr, "error: bad final value in trunc() test ( rf == %s )\n",
-                    rf.to_decimal( 1, -100 ).c_str() ); retval = 1; }
+                    rf.to_decimal( 1, -100 ).c_str() ); fail = true; }
   }
 
   // addition loop and parse test
@@ -554,21 +623,21 @@ int run_tests( const int level, const bool verbose )
     if( !r1.parse( s.c_str() ) || r1 != r )
       { std::fprintf( stderr, "error: parse error ( r == %s, r1 == %s )\n",
                       s.c_str(), r1.to_decimal( 1, -100 ).c_str() );
-        retval = 1; }
+        fail = true; }
     if( r == 0 ) zero = true;
     sum += r;
     r += ra;
     if( !( r1 < r ) )
       { std::fprintf( stderr, "error: comparison error ( %s < %s )\n",
                       s.c_str(), r.to_decimal( 1, -100 ).c_str() );
-        retval = 1; }
+        fail = true; }
     }
   if( sum != 0 )
     { std::fprintf( stderr, "error: sum is not zero ( sum == %s )\n",
-                    sum.to_decimal( 1, -100 ).c_str() ); retval = 1; }
+                    sum.to_decimal( 1, -100 ).c_str() ); fail = true; }
   if( !zero )
     { std::fputs( "error: zero not reached in sum loop.\n", stderr );
-      retval = 1; }
+      fail = true; }
   }
 
   // multiplication loop and parse test
@@ -581,32 +650,57 @@ int run_tests( const int level, const bool verbose )
     if( !r1.parse( s.c_str() ) || r1 != r )
       { std::fprintf( stderr, "error: parse error ( r == %s, r1 == %s )\n",
                       s.c_str(), r1.to_decimal( 1, -100 ).c_str() );
-        retval = 1; }
+        fail = true; }
     if( r == 1 ) one = true;
     r *= rm;
     if( r.error() && r1 < 149011 )
       { std::fprintf( stderr, "error: early overflow in mul loop ( r1 == %s )\n",
-                      r1.to_decimal( 1, -100 ).c_str() ); retval = 1; }
+                      r1.to_decimal( 1, -100 ).c_str() ); fail = true; }
     }
   if( !one )
     { std::fputs( "error: one not reached in mul loop.\n", stderr );
-      retval = 1; }
+      fail = true; }
   }
 
+  // parse boundary cases test
+  if( sizeof (int) == 4 )
+    {
+    Rational r;
+    if( !r.parse( "2147483647" ) || r != 2147483647 )
+      { std::fprintf( stderr, "error: parse error ( r == %s != 2147483647 )\n",
+                      r.to_decimal( 1, -100 ).c_str() ); fail = true; }
+    if( !(r=0).parse( "-2147483647" ) || r != -2147483647 )
+      { std::fprintf( stderr, "error: parse error ( r == %s != -2147483647 )\n",
+                      r.to_decimal( 1, -100 ).c_str() ); fail = true; }
+    if( !(r=0).parse( "0.0000000005" ) || r != Rational( 1, 2000000000 ) )
+      { std::fprintf( stderr, "error: parse error ( r == %s != 0.0000000005 )\n",
+                      r.to_decimal( 1, -100 ).c_str() ); fail = true; }
+    if( !(r=0).parse( "-0.0000000005" ) || r != Rational( -1, 2000000000 ) )
+      { std::fprintf( stderr, "error: parse error ( r == %s != -0.0000000005 )\n",
+                      r.to_decimal( 1, -100 ).c_str() ); fail = true; }
+    if( !(r=0).parse( "1/2147483647" ) || r != Rational( 1, 2147483647 ) )
+      { std::fprintf( stderr, "error: parse error ( r == %s != 1/2147483647 )\n",
+                      r.to_fraction().c_str() ); fail = true; }
+    if( !(r=0).parse( "-1/2147483647" ) || r != Rational( -1, 2147483647 ) )
+      { std::fprintf( stderr, "error: parse error ( r == %s != -1/2147483647 )\n",
+                      r.to_fraction().c_str() ); fail = true; }
+    }
+
   // parse error test
   {
-  Rational r( 123, 456 );		// 41/152
+  Rational r( 123, 456 );				// 41 / 152
   if( r.parse( "789/0" ) || r.parse( "789." ) || r.parse( ".-789" ) ||
-      r.parse( "789/%" ) || r.parse( "789.%" ) )
+      r.parse( "789/%" ) || r.parse( "789.%" ) ||
+      ( sizeof (int) == 4 && r.parse( "2147483648" ) ) )
     { std::fprintf( stderr, "error: unexpected parse() success ( r == %s )\n",
-                    r.to_fraction().c_str() ); retval = 1; }
+                    r.to_fraction().c_str() ); fail = true; }
   else if( r != Rational( 123, 456 ) )
     { std::fprintf( stderr, "error: parse() modified value on error ( r == %s )\n",
-                    r.to_fraction().c_str() ); retval = 1; }
+                    r.to_fraction().c_str() ); fail = true; }
   }
 
-  if( verbose && retval == 0 ) std::fputs( "Test passed.\n", stderr );
-  return retval;
+  if( verbosity >= 1 && !fail ) std::fputs( "Test passed.\n", stderr );
+  return !fail;
   }
 
 } // end namespace
@@ -614,15 +708,17 @@ int run_tests( const int level, const bool verbose )
 
 int main( const int argc, const char * const argv[] )
   {
+  const std::string * a_target = 0, * s_target = 0;
+  const char * a_pn = 0, * s_pn = 0;
   int level = 0;
   Rational sqrt_target;
   bool gr_flag = false;
   bool pi_flag = false;
-  bool verbose = false;
   if( argc > 0 ) invocation_name = argv[0];
 
   const Arg_parser::Option options[] =
     {
+    { 'a', "approximate",  Arg_parser::yes },
     { 'g', "golden-ratio", Arg_parser::no  },
     { 'h', "help",         Arg_parser::no  },
     { 'p', "pi",           Arg_parser::no  },
@@ -631,7 +727,7 @@ int main( const int argc, const char * const argv[] )
     { 't', "test",         Arg_parser::yes },
     { 'v', "verbose",      Arg_parser::no  },
     { 'V', "version",      Arg_parser::no  },
-    {  0 , 0,              Arg_parser::no  } };
+    { 0, 0,                Arg_parser::no  } };
 
   const Arg_parser parser( argc, argv, options );
   if( !parser.error().empty() )				// bad option
@@ -642,28 +738,31 @@ int main( const int argc, const char * const argv[] )
     {
     const int code = parser.code( argind );
     if( !code ) break;					// no more options
-    const char * const arg = parser.argument( argind ).c_str();
+    const char * const pn = parser.parsed_name( argind ).c_str();
+    const std::string & sarg = parser.argument( argind );
+    const char * const arg = sarg.c_str();
     switch( code )
       {
+      case 'a': a_target = &sarg; a_pn = pn; break;
       case 'g': gr_flag = true; break;
       case 'h': show_help(); return 0;
       case 'p': pi_flag = true; break;
-      case 'q': verbose = false; break;
-      case 's': if( !sqrt_target.parse( arg ) || sqrt_target <= 0 )
-                  { show_error( "Bad sqrt argument", 0, true ); return 1; }
-                break;
-      case 't': if( std::isdigit( *arg ) && *arg != '0' && !arg[1] )
-                  level = *arg - '0';
-                else { show_error( "Bad test level", 0, true ); return 1; }
-                break;
-      case 'v': verbose = true; break;
+      case 'q': verbosity = -1; break;
+      case 's': if( sqrt_target.parse( arg ) != (int)sarg.size() ||
+                    sqrt_target <= 0 )
+                  { show_option_error( arg, "Invalid sqrt target in", pn );
+                    return 1; } s_target = &sarg; s_pn = pn; break;
+      case 't': if( !std::isdigit( *arg ) || *arg == '0' || arg[1] )
+                  { show_option_error( arg, "Invalid test level in", pn );
+                    return 1; } level = *arg - '0'; break;
+      case 'v': if( verbosity < 4 ) ++verbosity; break;
       case 'V': show_version(); return 0;
       default: internal_error( "uncaught option." );
       }
     } // end process options
 
   int retval = 0;
-  if( level > 0 ) retval = run_tests( level, verbose );
+  if( level > 0 && !run_tests( level ) ) retval = 1;
 
   for( int i = 0; argind + i < parser.arguments(); ++i )
     {
@@ -677,8 +776,13 @@ int main( const int argc, const char * const argv[] )
         retval = 1; }
     }
 
+  if( a_target && !approximate_decimal( *a_target ) )
+    { show_option_error( a_target->c_str(), "Invalid target in", a_pn );
+      retval = 1; }
   if( gr_flag ) approximate_golden_ratio();
   if( pi_flag ) approximate_pi();
-  if( sqrt_target > 0 ) approximate_sqrt( sqrt_target );
+  if( s_target && !approximate_sqrt( sqrt_target ) )
+    { show_option_error( s_target->c_str(), "sqrt target out of range in", s_pn );
+      retval = 1; }
   return retval;
   }
